from pythonit_toolkit.starlette_backend.pastaporto_backend import (
    PastaportoAuthBackend,
    on_auth_error,
)
from starlette.middleware import Middleware
from starlette.middleware.authentication import AuthenticationMiddleware


def pastaporto_auth_middleware(secret: str):
    return Middleware(
        AuthenticationMiddleware,
        backend=PastaportoAuthBackend(secret),
        on_error=on_auth_error,
    )



from pythonit_toolkit.headers import PASTAPORTO_X_HEADER
from pythonit_toolkit.pastaporto.entities import Pastaporto, RequestAuth
from pythonit_toolkit.pastaporto.exceptions import InvalidPastaportoError
from sentry_sdk import set_user
from starlette.authentication import AuthenticationBackend, AuthenticationError
from starlette.responses import JSONResponse
from starlette.routing import request_response


def on_auth_error(request: request_response, exc: Exception):
    return JSONResponse({"errors": [{"message": str(exc)}]}, status_code=401)


class PastaportoAuthBackend(AuthenticationBackend):
    def __init__(self, secret: str) -> None:
        super().__init__()
        self.pastaporto_secret = secret

    async def authenticate(self, request):
        if PASTAPORTO_X_HEADER not in request.headers:
            # TODO: Always fail request without pastaporto?
            set_user(None)
            return

        pastaporto_token = request.headers[PASTAPORTO_X_HEADER]

        try:
            pastaporto = Pastaporto.from_token(pastaporto_token, self.pastaporto_secret)

            if pastaporto.is_authenticated:
                set_user({"id": pastaporto.user_info.id, "ip_address": "{{auto}}"})
            else:
                set_user(None)

            return RequestAuth(pastaporto), pastaporto.user_info
        except InvalidPastaportoError as e:
            set_user(None)
            raise AuthenticationError("Invalid pastaporto") from e



from datetime import datetime, timedelta, timezone

import jwt
from pythonit_toolkit.pastaporto.entities import Credential


def fake_pastaporto_token_for_user(
    user: dict[str, str], secret: str, *, staff: bool = False
):
    credentials = [Credential.AUTHENTICATED]
    if staff:
        credentials.append(Credential.STAFF)

    now = datetime.now(timezone.utc)
    return jwt.encode(
        {
            "user_info": {"id": user["id"], "email": user["email"], "is_staff": staff},
            "credentials": credentials,
            "exp": now + timedelta(minutes=1),
            "iat": now,
            "iss": "users",
        },
        secret,
        algorithm="HS256",
    )


def fake_service_to_service_token(secret: str, *, issuer: str, audience: str):
    now = datetime.now(timezone.utc)
    return jwt.encode(
        {
            "iss": issuer,
            "aud": audience,
            "exp": now + timedelta(minutes=1),
            "iat": now,
        },
        secret,
        algorithm="HS256",
    )



import json
import logging
from collections import namedtuple
from dataclasses import dataclass
from typing import Any, Dict, Optional

from pythonit_toolkit.headers import PASTAPORTO_X_HEADER, SERVICE_JWT_HEADER
from pythonit_toolkit.pastaporto.test import (
    fake_pastaporto_token_for_user,
    fake_service_to_service_token,
)

logger = logging.getLogger(__name__)


SimulatedUser = namedtuple("User", ["id", "email", "is_staff"])


@dataclass
class Response:
    errors: Optional[Dict[str, Any]]
    data: Optional[Dict[str, Any]]


class GraphQLClient:
    def __init__(
        self,
        client,
        *,
        pastaporto_secret: Optional[str] = None,
        service_to_service_secret: Optional[str] = None,
        internal_api_endpoint: bool = False
    ):
        self._client = client
        self._pastaporto_secret = pastaporto_secret
        self._service_to_service_secret = service_to_service_secret
        self.pastaporto_token = None
        self.service_to_service_token = None

        if internal_api_endpoint:
            self.endpoint = "/internal-api"
        else:
            self.endpoint = "/graphql"

    async def query(
        self,
        query: str,
        variables: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, Any]] = None,
    ) -> Response:
        body = {"query": query}
        headers = headers or {}

        if variables:
            body["variables"] = variables

        if self.pastaporto_token:
            headers[PASTAPORTO_X_HEADER] = self.pastaporto_token

        if self.service_to_service_token:
            headers[SERVICE_JWT_HEADER] = self.service_to_service_token

        resp = await self._client.post(self.endpoint, json=body, headers=headers)
        data = json.loads(resp.content.decode())
        return Response(errors=data.get("errors"), data=data.get("data"))

    def force_login(self, user: SimulatedUser):
        self.pastaporto_token = fake_pastaporto_token_for_user(
            {"id": user.id, "email": user.email},
            str(self._pastaporto_secret),
            staff=user.is_staff,
        )

    def force_service_login(
        self,
        issuer: str = "gateway",
        audience: str = "users-backend",
        key: Optional[str] = None,
    ):
        self.service_to_service_token = fake_service_to_service_token(
            str(key or self._service_to_service_secret),
            issuer=issuer,
            audience=audience,
        )



from typing import Dict, List, Optional

from pythonit_toolkit.emails.templates import EmailTemplate

from .base import EmailBackend


class LocalEmailBackend(EmailBackend):
    def __init__(self, environment: Optional[str] = None) -> None:
        super().__init__(environment=environment)

    def send_email(
        self,
        *,
        template: EmailTemplate,
        subject: str,
        from_: str,
        to: str,
        variables: Optional[Dict[str, str]] = None,
        reply_to: List[str] = None,
    ):
        reply_to = reply_to or []

        print("=== Email sending ===")
        print(f"Template: {template}")
        print(f"From: {from_}")
        print(f"To: {to}")
        print(f"Subject: {subject}")
        print(f"Variables: {str(variables)}")
        print(f"Reply to: {str(reply_to)}")
        print("=== End Email sending ===")



import json
from typing import Dict, List, Optional
import html
import boto3
from pythonit_toolkit.emails.templates import EmailTemplate
from pythonit_toolkit.emails.utils import SafeString

from .base import EmailBackend


class SESEmailBackend(EmailBackend):
    def __init__(self, environment: str) -> None:
        super().__init__(environment)
        self.ses = boto3.client("ses")

    def send_email(
        self,
        *,
        template: EmailTemplate,
        subject: str,
        from_: str,
        to: str,
        variables: Optional[Dict[str, str]] = None,
        reply_to: List[str] = None,
    ):
        reply_to = reply_to or []

        variables = self.encode_vars({"subject": subject, **(variables or {})})
        self.ses.send_templated_email(
            Source=from_,
            Destination={"ToAddresses": [to]},
            Template=f"pythonit-{self.environment}-{template}",
            TemplateData=json.dumps(variables),
            ReplyToAddresses=reply_to,
            ConfigurationSetName='primary',
        )

    def encode_vars(self, variables: dict[str, str]) -> dict[str, str]:
        vars = dict()

        for key, value in variables.items():
            if not isinstance(value, SafeString):
                value = html.escape(value)

            vars[key] = str(value)

        return vars
